let
  pkgs = import <nixpkgs> { };

  gridlib = import ../../lib;
  rawConfig = pkgs.lib.trivial.importJSON ./config.json;
  config = rawConfig // {
    sshUsers = import ./public-keys/users.nix;

    # Convert relative paths to absolute so library code can resolve names
    # correctly.
    publicKeyPath = toString ./. + "/${rawConfig.publicKeyPath}";
    privateKeyPath = toString ./. + "/${rawConfig.privateKeyPath}";
  };

  # Configure deployment management authorization for all systems in the grid.
  grid-config = {
    imports = [
      gridlib.base
    ];
    services.private-storage.deployment = {
      authorizedKey = builtins.readFile "${config.publicKeyPath}/deploy_key.pub";
      gridName = "local";
    };
    services.private-storage.sshUsers = config.sshUsers;
    grid = {
      inherit (config) publicKeyPath privateKeyPath; 
    };
  };

  payments = {
    imports = [
      gridlib.issuer
      gridlib.hardware-virtual
      (gridlib.customize-issuer (config // {
          monitoringvpnIPv4 = "172.23.23.11";
      }))
      grid-config
    ];
    hardware-virtual.publicIPv4 = "192.168.67.21";
  };

  storage1 = {
    imports = [
      gridlib.storage
      gridlib.hardware-virtual
      (gridlib.customize-storage (config // {
        monitoringvpnIPv4 = "172.23.23.12";
      }))
      grid-config
    ];
    hardware-virtual.publicIPv4 = "192.168.67.22";
    system.stateVersion = "19.09";
  };

  storage2 = {
    imports = [
      gridlib.storage
      gridlib.hardware-virtual
      (gridlib.customize-storage (config // {
        monitoringvpnIPv4 = "172.23.23.13";
      }))
      grid-config
    ];
    hardware-virtual.publicIPv4 = "192.168.67.23";
    system.stateVersion = "19.09";
  };

  monitoring = {
    imports = [
      gridlib.monitoring
      gridlib.hardware-virtual
      (gridlib.customize-monitoring {
        inherit paymentExporterTargets;
        inherit (config) domain letsEncryptAdminEmail;
        googleOAuthClientID = config.monitoringGoogleOAuthClientID;
        monitoringvpnIPv4 = "172.23.23.1";
      })
      grid-config
    ];
    hardware-virtual.publicIPv4 = "192.168.67.24";
    system.stateVersion = "19.09";
  };

  # TBD: derive these automatically:
  paymentExporterTargets = [ "payments" ];

in {
  network = {
    description = "PrivateStorage.io LocalDev Grid";
  };
  inherit payments monitoring storage1 storage2;
}
